﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using SPIDLL2;

namespace ConsoleApplication1
{

    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Starting Test1: Class Interface spidll2\n");
            // Instantiate class with constructor
            // Trace -1, OpFSel 1 for Testmode 0 for BFS Key
            SPIDLL2.spidll2 crypto = new SPIDLL2.spidll2(-1,1);

            // Get encrypted Session Key (CKey)
            String sessionKey1 = crypto.GetCKey();
            
            // Write MX Record with CKey
            String mxline = "MX|60619008|KLI1|19980103|1|19970101|19970201|1|" + sessionKey1 + "|\n";
            Console.WriteLine(mxline);
            // Generate Hans Meier Testrecord
            String encr_vbcode = crypto.EncryptPerson(@"Meier", @"Hans", @"19131211", @"1");
            // Write Hans Meier MB line
            String meierline = String.Format("MB||||{0}||||||||||||||||||||||||||||||||||||||||||||||\n", encr_vbcode);
            Console.WriteLine(meierline);

            // Example of real person
            String encr_vbcode2 = crypto.EncryptPerson(@"Hürzeler", @"Alfred", @"19720916", @"1");
            // Example of encryped CHOP Code
            String code2 = crypto.EncryptChopIC(@"A125", 1);
            // Write person MB record with encrypted CHOP code
            String mbline2 = String.Format("MB||||{0}|A|1|0|0|0||||||||||||||||||||{1}|||||||||||fld41||||||||||\n", encr_vbcode2, code2);
            Console.WriteLine(mbline2);

            // Example of real person
            String encr_vbcode3 = crypto.EncryptPerson(@"Kunz", @"Hans", @"19331011", @"1");
            // Example of encryped CHOP Code
            String code3 = crypto.EncryptChopIC(@"D2", 2);
            // Write person MB record with encrypted CHOP code
            String mbline3 = String.Format("MB||||{0}|A|1|0|0|0|||||||||||||||||||||{1}||||||||||fld41||||||||||\n", encr_vbcode3, code3);
            Console.WriteLine(mbline2);

            // Example of real person
            String encr_vbcode4 = crypto.EncryptPerson(@"Hintz", @"Fritz", @"19520121", @"1");
            // Example of encryped CHOP Code
            String code4 = crypto.EncryptChopIC(@"D2", 2);
            // Write person MB record with encrypted CHOP code
            String mbline4 = String.Format("MB||||{0}|A|1|0|0|0|||||||||||||||||||||{1}||||||||||fld41||||||||||\n", encr_vbcode4, code4);
            Console.WriteLine(mbline4);

            // Example of real person
            String encr_vbcode5 = crypto.EncryptPerson(@"Berger", @"Eva", @"19810702", @"2");
            // Example of encryped CHOP Code
            String code5 = crypto.EncryptChopIC(@"D2", 2);
            // Write person MB record with encrypted CHOP code
            String mbline5 = String.Format("MB||||{0}|A|1|0|0|0|||||||||||||||||||||{1}||||||||||fld41||||||||||\n", encr_vbcode5, code5);
            Console.WriteLine(mbline5);

            // Example of anonymization of an ID instead of Name, Surname, Birthdate, Sex
            String encr_id1 = crypto.EncryptId(@"P12760");
            String idline1 = String.Format("MB||||{0}||ID|not|Name||||||||||||||||||||||||||||||||||||||||||\n", encr_id1);
            Console.WriteLine(idline1);

            // Example of anonymization of an ID instead of Name, Surname, Birthdate, Sex
            String encr_id2 = crypto.EncryptId(@"abc45678901234567890123456789012");
            String idline2 = String.Format("MB||||{0}||ID|not|Name||||||||||||||||||||||||||||||||||||||||||\n", encr_id2);
            Console.WriteLine(idline2);

            // Example of anonymization of an ID instead of Name, Surname, Birthdate, Sex
            String encr_id3 = crypto.EncryptId(@"abc45678901234567890123456789013");
            String idline3 = String.Format("MB||||{0}||ID|not|Name||||||||||||||||||||||||||||||||||||||||||", encr_id3);
            Console.WriteLine(idline3);

            Console.WriteLine("Now writing result into file test1.dat\n");
            using (FileStream test_file1 = File.Create(@"test1.dat"))
            {
                byte[] info = new ASCIIEncoding().GetBytes(mxline);
                test_file1.Write(info, 0, info.Length);

                info = new ASCIIEncoding().GetBytes(meierline);
                test_file1.Write(info, 0, info.Length);

                info = new ASCIIEncoding().GetBytes(mbline2);
                test_file1.Write(info, 0, info.Length);

                info = new ASCIIEncoding().GetBytes(mbline3);
                test_file1.Write(info, 0, info.Length);

                info = new ASCIIEncoding().GetBytes(mbline4);
                test_file1.Write(info, 0, info.Length);

                info = new ASCIIEncoding().GetBytes(mbline5);
                test_file1.Write(info, 0, info.Length);

                info = new ASCIIEncoding().GetBytes(idline1);
                test_file1.Write(info, 0, info.Length);

                info = new ASCIIEncoding().GetBytes(idline2);
                test_file1.Write(info, 0, info.Length);

                info = new ASCIIEncoding().GetBytes(idline3);
                test_file1.Write(info, 0, info.Length);

                test_file1.Close();
            }
          
            Console.WriteLine("Test1 finished.\n");


            Console.WriteLine("Starting Test2: Batch interface\n");
            // New Session Key, for testing
            String sessionKey2 = crypto.NewCKey();
            crypto.MXEncrypt(@"test.inp",
                             @"test2.dat", 1, 1);
            Console.WriteLine("Finished Test2.\n");

            Console.WriteLine("Starting Test3: Decryption\n");
            // Decrypting encrypted CHOP code.
            String msg = new String(' ', 256);

            Console.WriteLine("Reencrypting Hashcode to Verbindungscode...\n");
            // pass session key to first call to a decryption method
            String vbcode = crypto.DecryptHashCode(sessionKey1, @"00010002", encr_vbcode, 0, msg);
            Console.WriteLine("VB Code (reencrypted) is {0}\n", vbcode);

            Console.WriteLine("Decrypting encrypted CHOP code...\n");
            // Successive decrypt calls don't need session key parameter (uses last session key)
            String orig_code = crypto.DecryptField(@"", @"00010002", code2, 0, msg);
            Console.WriteLine("Decryted CHOP code is {0}\n", orig_code);

            Console.WriteLine("Finished Test3. Press a key");
            Console.Read();
        }
    }
}
